/*
 * roomba.c
 *
 *  Created on: 4-Feb-2009
 *      Author: nrqm
 */

#include <util/delay.h>
#include "uart.h"
#include "roomba.h"
#include "roomba_sci.h"

void Roomba_Init()
{

	// Try 57.6 kbps to start (this is the Roomba's default baud rate after the battery is installed).
	//uart_init(UART_57600);

	// Try to start the SCI
	uart_write_byte(START);
	_delay_ms(20);


	//roomba_lower_baud_rate();
	// change the baud rate.  Have to wait for 100 ms after changing the baud rate.
	uart_write_byte(BAUD);
	uart_write_byte(ROOMBA_19200BPS);
	_delay_ms(100);		

	uart_init(UART_19200);
	// start the SCI again in case the first start didn't go through.
	uart_write_byte(START);
	_delay_ms(20);

	// finally put the Roomba into safe mode.
	uart_write_byte(CONTROL);
	_delay_ms(20);
}

void Roomba_UpdateSensorPacket(ROOMBA_SENSOR_GROUP group, roomba_sensor_data_t* sensor_packet)
{
	
	uart_write_byte(SENSORS);
	uart_write_byte(group);
	
	switch(group)
	{
	case EXTERNAL:
	
		// environment sensors
		while (uart_bytes_received() != 10);
		
		sensor_packet->bumps_wheeldrops = uart_read_byte(0);
		sensor_packet->wall = uart_read_byte(1);
		sensor_packet->cliff_left = uart_read_byte(2);
		sensor_packet->cliff_front_left = uart_read_byte(3);
		sensor_packet->cliff_front_right = uart_read_byte(4);
		sensor_packet->cliff_right = uart_read_byte(5);
		sensor_packet->virtual_wall = uart_read_byte(6);
		sensor_packet->motor_overcurrents = uart_read_byte(7);
		sensor_packet->dirt_left = uart_read_byte(8);
		sensor_packet->dirt_right = uart_read_byte(9);
		break;
	case CHASSIS:
		// chassis sensors
		while (uart_bytes_received() != 6);
		sensor_packet->remote_opcode = uart_read_byte(0);
		sensor_packet->buttons = uart_read_byte(1);
		sensor_packet->distance.bytes.high_byte = uart_read_byte(2);
		sensor_packet->distance.bytes.low_byte = uart_read_byte(3);
		sensor_packet->angle.bytes.high_byte = uart_read_byte(4);
		sensor_packet->angle.bytes.low_byte = uart_read_byte(5);
		break;
	case INTERNAL:
		// internal sensors
		while (uart_bytes_received() != 10);
		sensor_packet->charging_state = uart_read_byte(0);
		sensor_packet->voltage.bytes.high_byte = uart_read_byte(1);
		sensor_packet->voltage.bytes.low_byte = uart_read_byte(2);
		sensor_packet->current.bytes.high_byte = uart_read_byte(3);
		sensor_packet->current.bytes.low_byte = uart_read_byte(4);
		sensor_packet->temperature = uart_read_byte(5);
		sensor_packet->charge.bytes.high_byte = uart_read_byte(6);
		sensor_packet->charge.bytes.low_byte = uart_read_byte(7);
		sensor_packet->capacity.bytes.high_byte = uart_read_byte(8);
		sensor_packet->capacity.bytes.low_byte = uart_read_byte(9);
		break;
	}
	uart_reset_receive();
}

void Roomba_Drive( int16_t velocity, int16_t radius )
{
	uart_write_byte(DRIVE);
	uart_write_byte(HIGH_BYTE(velocity));
	uart_write_byte(LOW_BYTE(velocity));
	uart_write_byte(HIGH_BYTE(radius));
	uart_write_byte(LOW_BYTE(radius));
}

