/*
 * uart.c
 *
 * Created: 5/28/2013 3:56:25 PM
 *  Author: rayhan
 */ 

#include "uart.h"
#include <avr/interrupt.h>

static volatile uint8_t uart_buffer[UART_BUFFER_SIZE];
static volatile uint8_t uart_buffer_index;

ISR(USART1_RX_vect)
{
	uart_buffer[uart_buffer_index] = UDR1;
	uart_buffer_index = (uart_buffer_index + 1) % UART_BUFFER_SIZE;
	
}


void uart_init(UART_BPS bitrate)
{

	UCSR1B = _BV(RXEN1) | _BV(TXEN1) | _BV(RXCIE1);
	UCSR1C = _BV(UCSZ10) | _BV(UCSZ11);
	UBRR1H = 0;

	//for atmel 1280, see page 231 of the datasheet
	switch (bitrate)
	{
		case  UART_19200:
		UBRR1L = 51;
		break;
		case UART_38400:
		UBRR1L = 25;
		break;
		case UART_57600:
		UBRR1L = 16;
		break;
	}
	
	uart_buffer_index = 0;
	
}

void uart_write_byte(uint8_t byte)
{
	while (!(UCSR1A & (1 << UDRE1)));
	UDR1 = byte;
}


uint8_t uart_read_byte(int index)
{
	if (index < UART_BUFFER_SIZE)
	{
		return uart_buffer[index];
	}
	return 0;
}

uint8_t uart_bytes_received(void)
{
	return uart_buffer_index;
}

void uart_reset_receive(void)
{
	uart_buffer_index = 0;
}


